@@ -1,7 +1,7 @@
 # See t/plugin_new_method_backcompat.t
 package TestAppPluginWithConstructor;
 use Test::More;
-use Test::Exception;
+use Test::Fatal;
 use Catalyst qw/+TestPluginWithConstructor/;
 use Moose;
 extends qw/Catalyst/;
@@ -9,9 +9,9 @@ extends qw/Catalyst/;
 __PACKAGE__->setup;
 our $MODIFIER_FIRED = 0;
 
-lives_ok {
+is exception {
     before 'dispatch' => sub { $MODIFIER_FIRED = 1 }
-} 'Can apply method modifier';
+}, undef, 'Can apply method modifier';
 no Moose;
 
 our $IS_IMMUTABLE_YET = __PACKAGE__->meta->is_immutable;
